@@ -124,12 +124,12 @@ public:
         QLibrarySettings *ls = qt_library_settings();
         if (ls) {
 #ifndef QT_BUILD_QMAKE
-            if (ls->reloadOnQAppAvailable && QCoreApplication::instance() != 0)
+            if (ls->reloadOnQAppAvailable && QCoreApplication::instance() != nullptr)
                 ls->load();
 #endif
             return ls->settings.data();
         } else {
-            return 0;
+            return nullptr;
         }
     }
 };
@@ -146,7 +146,7 @@ void QLibrarySettings::load()
     // If we get any settings here, those won't change when the application shows up.
     settings.reset(QLibraryInfoPrivate::findConfiguration());
 #ifndef QT_BUILD_QMAKE
-    reloadOnQAppAvailable = (settings.data() == 0 && QCoreApplication::instance() == 0);
+    reloadOnQAppAvailable = (settings.data() == nullptr && QCoreApplication::instance() == nullptr);
     bool haveDevicePaths;
     bool haveEffectivePaths;
     bool havePaths;
@@ -169,7 +169,7 @@ void QLibrarySettings::load()
                     || children.contains(QLatin1String("Paths"));
 #ifndef QT_BUILD_QMAKE
         if (!havePaths)
-            settings.reset(0);
+            settings.reset(nullptr);
 #else
     } else {
         haveDevicePaths = false;
@@ -212,7 +212,7 @@ QSettings *QLibraryInfoPrivate::findConfiguration()
             return new QSettings(qtconfig, QSettings::IniFormat);
     }
 #endif
-    return 0;     //no luck
+    return nullptr;     //no luck
 }
 
 #endif // settings
@@ -535,8 +535,25 @@ static QString getRelocatablePrefix()
 #if defined(QT_STATIC)
     prefixPath = prefixFromAppDirHelper();
 #elif defined(Q_OS_DARWIN) && QT_CONFIG(framework)
-    CFBundleRef qtCoreBundle = CFBundleGetBundleWithIdentifier(
-            CFSTR("org.qt-project.QtCore"));
+    auto qtCoreBundle = CFBundleGetBundleWithIdentifier(CFSTR("org.qt-project.QtCore"));
+    if (!qtCoreBundle) {
+        // When running Qt apps over Samba shares, CoreFoundation will fail to find
+        // the Resources directory inside the bundle, This directory is a symlink,
+        // and CF relies on readdir() and dtent.dt_type to detect symlinks, which
+        // does not work reliably for Samba shares. We work around it by manually
+        // looking for the QtCore bundle.
+        auto allBundles = CFBundleGetAllBundles();
+        auto bundleCount = CFArrayGetCount(allBundles);
+        for (int i = 0; i < bundleCount; ++i) {
+            auto bundle = CFBundleRef(CFArrayGetValueAtIndex(allBundles, i));
+            auto url = QCFType<CFURLRef>(CFBundleCopyBundleURL(bundle));
+            auto path = QCFType<CFStringRef>(CFURLCopyFileSystemPath(url, kCFURLPOSIXPathStyle));
+            if (CFStringHasSuffix(path, CFSTR("/QtCore.framework"))) {
+                qtCoreBundle = bundle;
+                break;
+            }
+        }
+    }
     Q_ASSERT(qtCoreBundle);
 
     QCFType<CFURLRef> qtCorePath = CFBundleCopyBundleURL(qtCoreBundle);
@@ -644,7 +661,7 @@ static QString getHostPrefixFromHostBinDir()
 #endif
 
 #ifndef QT_BUILD_QMAKE_BOOTSTRAP
-static const char *getPrefix(
+static QString getPrefix(
 #ifdef QT_BUILD_QMAKE
         QLibraryInfo::PathGroup group
 #endif
@@ -653,15 +670,13 @@ static const char *getPrefix(
 #if defined(QT_BUILD_QMAKE)
 #  if QT_CONFIGURE_CROSSBUILD
     if (group == QLibraryInfo::DevicePaths)
-        return QT_CONFIGURE_PREFIX_PATH;
+        return QString::fromLocal8Bit(QT_CONFIGURE_PREFIX_PATH);
 #  endif
-    static QByteArray extPrefixPath = getExtPrefixFromHostBinDir().toLatin1();
-    return extPrefixPath.constData();
+    return getExtPrefixFromHostBinDir();
 #elif QT_CONFIG(relocatable)
-    static QByteArray prefixPath = getRelocatablePrefix().toLatin1();
-    return prefixPath.constData();
+    return getRelocatablePrefix();
 #else
-    return QT_CONFIGURE_PREFIX_PATH;
+    return QString::fromLocal8Bit(QT_CONFIGURE_PREFIX_PATH);
 #endif
 }
 #endif // QT_BUILD_QMAKE_BOOTSTRAP
@@ -775,9 +790,14 @@ QLibraryInfo::rawLocation(LibraryLocation loc, PathGroup group)
 
 #ifndef QT_BUILD_QMAKE_BOOTSTRAP
     if (!fromConf) {
-        const char * volatile path = 0;
+        // "volatile" here is a hack to prevent compilers from doing a
+        // compile-time strlen() on "path". The issue is that Qt installers
+        // will binary-patch the Qt installation paths -- in such scenarios, Qt
+        // will be built with a dummy path, thus the compile-time result of
+        // strlen is meaningless.
+        const char * volatile path = nullptr;
         if (loc == PrefixPath) {
-            path = getPrefix(
+            ret = getPrefix(
 #ifdef QT_BUILD_QMAKE
                         group
 #endif
